/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, builtinclass, uuid(57ac3989-531e-4928-998b-53fde81a05a6)]
interface nsIDOMMozSmsSegmentInfo : nsISupports
{
  /* The number of total segments for the input string. */
  readonly attribute long segments;
  /* The number of characters available per segment. */
  readonly attribute long charsPerSegment;
  /* The maximum number of available characters in the last segment. */
  readonly attribute long charsAvailableInLastSegment;
};
